function [mse_set] = alterMin_DC(ch, params)
hd = ch.hd;
Gd = ch.Gd;
gd = ch.gd;
hu = ch.hu;
Gu = ch.Gu;
gu = ch.gu;

fu = ch.fu;
fd = ch.fd;

noise = params.noise;
pmax = params.pmax;
iter_max = params.iter_max;

[~, K] = size(ch.gu);
%%
mse_set = nan(iter_max, 1);
for i = 1:iter_max
    if params.verb
        fprintf('Solving... %d/%d.\n', i, iter_max);
    end
    %% aggregation beamforming vector
    if params.verb
       fprintf('=== iter: %d ==== find v ===\n',i);
    end
    
    [v0, ~, ~, Rv] = find_V_SDR(ch, fd, fu, pmax);
    [v, mse, feasibleV] = find_V_DC(ch, fd, fu, params, v0, Rv);
    mse_set(i) = mse*noise;
%     mse_set(i)
    if i > 1 && abs(mse_set(i) - mse_set(i-1)) < 1e-4
        break;
    end
    if feasibleV == 0
        mse_set = nan;
        if params.verb
            fprintf('The solution v_dc is infeasible.\n');
        end
        break;
    end
	%% uplink phase-shift
    if params.verb
        fprintf('=== iter: %d ==== find p ===\n',i)
    end
    [thetaU, feasibleP] = find_P_SDR(v, ch, fd, fu, pmax);
    if feasibleP == 0
        [thetaU, feasibleP] = find_P_DC(v, ch, params, fd, fu);
    end
    if feasibleP == 0
%         mse_set = nan;
        if params.verb
            fprintf('The solution p_dc is infeasible.\n');
        end
        break;
    else
        ThetaU = diag(thetaU);
        fu = nan(size(gu));
        for k = 1:K
            fu(:, k) = Gu*ThetaU*hu(:, k)+gu(:, k);
        end
    end
	%% downlink phase-shift
    if params.verb
        fprintf('=== iter: %d ==== find q ===\n',i)
    end
    
    [thetaD, feasibleQ] = find_Q_SDR(v, ThetaU, ch, fd, fu, pmax);
    if feasibleQ == 0
        [thetaD, feasibleQ] = find_Q_DC(v, ThetaU, ch, params, fd, fu);
    end
    if feasibleQ == 0
%         mse_set = nan;
        if params.verb
            fprintf('The solution q_dc is infeasible.\n');
        end
        break;
    else
        ThetaD = diag(thetaD);
        fd = nan(size(gd'));
        for k = 1:K
            fd(k, :) = hd(:, k)'*ThetaD*Gd+gd(:, k)';
        end
    end
end
end